(* reduced ordered binary decision diagrams *)

(* John D. Ramsdell -- Decemeber 1998 *)

(* Copyright (C) 2001 The MITRE Corporation

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *)

(* types *)

type bdd

(* predicates *)

val is_same : bdd -> bdd -> bool	(* are bdds the same? *)
val is_node : bdd -> bool		(* is bdd an internal node? *)
val is_truth : bdd -> bool		(* is bdd the truth leaf? *)
val is_falsehood : bdd -> bool		(* is bdd the falsehood leaf? *)

(* Hashing consistent with the is_same equivalence relation *)

val hash : bdd -> int

(* constructors *)

val truth : bdd
val falsehood : bdd
(* construct the boolean function f(x) = x; x must be non-negative *)
val identity : int -> bdd

(* node accessors *)

(* Failure is raised by these three functions when applied to a leaf *)

val variable : bdd -> int
val consequent : bdd -> bdd
val alternative : bdd -> bdd

(* bdd operators *)

val complement : bdd -> bdd
(* if-then-else, the bdd composition operator *)
val ite : bdd * bdd * bdd -> bdd

(* Hash table parameters *)

(* initialize the bdd hash tables with the following parameters *)
val init : int				(* number of unique tables *)
    * int				(* initial size of a table *)
    * int				(* number of computed tables *)
    * int				(* initial size of a table *)
    -> unit

type init_defaults = {
    utn : int;
    uts : int;
    ctn : int;
    cts : int;
  }

val init_defaults : init_defaults

(* Declare maximum number of variables *)

val set_var_num : int -> unit

(* Printer *)
val print_bdd : bdd -> unit
