(* some common operations on bdds and a simple printer *)

(* John D. Ramsdell -- Decemeber 1998 *)

(* Copyright (C) 2001 The MITRE Corporation

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *)

open Bdd

(* Paths are lists of pairs.  The second item in the pair is the
   variable and first item is a boolean value telling if the variable
   should be negated.  A variable is a non-negative integer.  A path
   is sorted by the integer in decreasing order.  A path may not
   contain two pairs with the same variable. *)

type path = (bool * int) list

val conjoin : bdd -> bdd -> bdd
val disjoin : bdd -> bdd -> bdd
val difference : bdd -> bdd -> bdd
val implies : bdd -> bdd -> bdd
(* prod i s f g computes the relational product of f and g
   over the variables v that satisfy i <= v < i + s. *)
val prod : int -> int -> bdd -> bdd -> bdd
(* rename i s b produces a bdd from b with variables i or greater
   renamed to be i + s.  Note that s can be negative. *)
val rename : int -> int -> bdd -> bdd
(* collapse i s b produces the bdd that contains only variable from
   i to i + s. *)
val collapse : int -> int -> bdd -> bdd
val path2bdd : path -> bdd
