(* $Id: lex_formulas.mll,v 1.1.1.1 2003/11/05 13:07:21 ramsdell Exp $ *)

{

open Parse_formulas

let line_number = ref 1
let line_start = ref 0
let newline_found lexbuf =
  line_number := !line_number + 1;
  line_start := Lexing.lexeme_end lexbuf
let init() =
  line_number := 1;
  line_start := 0
let lexeme_place lexbuf =
  let start =
    Lexing.lexeme_start lexbuf - !line_start + 1 in
  let finish =
    Lexing.lexeme_end lexbuf - !line_start + 1 in
  (!line_number, start, finish)

}

let start = ['A'-'Z' 'a'-'z' '_' '$']
let part = start | ['0'-'9']
let identifier = start part*
rule token = parse
  (* skip comments starting with the hash mark '#' *)
  '#' [^ '\n' ]* '\n'	    { newline_found lexbuf; token lexbuf }
| '\n'                      { newline_found lexbuf; token lexbuf }
| [' ' '\t' '\r']           { token lexbuf }
| '('                       { LPAREN }
| ')'                       { RPAREN }
| '['                       { LBRACK }
| ']'                       { RBRACK }
| '{'                       { LBRACE }
| '}'                       { RBRACE }
| '+'                       { PLUS }
| ','                       { COMMA }
| ';'                       { SEMICOLON }
| "->"                      { IMPLY }
| "<->"                     { IFF }
| '|'                       { OR }
| '&'                       { AND }
| '!'                       { NOT }
| '='                       { EQ }
| "!="                      { NEQ }
| ':'                       { IN }
| "!:"                      { NOTIN }
| "'"                       { NEXT }
| "c"                       { CLASSES }
| "p"                       { PERMISSIONS }
| "t"                       { TYPES }
| "r"                       { ROLES }
| "u"                       { USERS }
| "TRUE"                    { TRUE }
| "FALSE"                   { FALSE }
| "STATE"                   { STATE }
| "ACTION"                  { ACTION }
| "INIT"                    { INIT }
| "TRANS"                   { TRANS }
| "SPEC"                    { SPEC }
| identifier                { STRING(Lexing.lexeme lexbuf) }
| eof                       { EOF }
