(* This section is all for argument processing *)

(* Mode *)

let version = ref false

let ltl = ref false

(* file names *)

let output_file = ref "-"

let lts_file = ref "-"

let diagram_file = ref ""

let anon_var = ref 0

let anon_fun str =
  match !anon_var with
    0 ->
      lts_file := str;
      anon_var := 1
  | 1 ->
      diagram_file := str;
      anon_var := 2
  | _ ->
      anon_var := 3

(* set up Arg.parse *)

let arg_spec =
  [("-ltl", Arg.Set(ltl),
    "   - generate specifications in LTL");
   ("-ctl", Arg.Clear(ltl),
    "   - generate specifications in CTL (default)");
   ("-o", Arg.String(fun s -> output_file := s),
    "name - send output to this file (default stdout)" );
   ("-v", Arg.Set(version),
    "     - display version information and exit");
   ("--", Arg.Rest(anon_fun),
    "     - treat remaining args as file names, where - means stdin")]

let usage_msg = "Usage: lts2smv [options] [lts [diagrams]]";;

Arg.parse arg_spec anon_fun usage_msg;

if !version then
  begin
    Print_version.print_version();
    exit 0
  end;

if !anon_var > 2 then
  begin
    prerr_string "bad arg count";
    prerr_newline();
    prerr_string usage_msg;
    prerr_newline();
    exit 1
  end;

let lts_file_name = !lts_file in
let diagram_file_name = !diagram_file in
let output_file_name = !output_file in
let ltl_flag = !ltl in

if lts_file_name = "" && diagram_file_name = "" then
  begin
    prerr_string "no input files specified";
    prerr_newline();
    prerr_string usage_msg;
    prerr_newline();
    exit 1
  end;

(* read input *)

try

  let lts =
    if lts_file_name = "" then
      None
    else
      Some (Read_formulas.read_lts lts_file_name) in

  let diagrams =
    if diagram_file_name = "" then
      None
    else
      Some (Read_formulas.read_diagrams diagram_file_name) in

(* open output file *)

  let out =
    if output_file_name = "-" then
      stdout
    else
      open_out output_file_name in

  Format.set_formatter_out_channel out;

(* produce output *)

  begin
    match lts with
      Some lts -> Nusmv.nusmv ltl_flag lts;
    | None -> ();
  end;
  begin
    match diagrams with
      Some diagrams -> List.iter (Nusmv.emit_diagram ltl_flag) diagrams
    | None -> ();
  end;
  exit 0

with Failure s ->
  prerr_string s;
  prerr_newline();
  exit 1
