%{

open Formulas

%}

%token <string> STRING
%token CLASSES, PERMISSIONS, TYPES, ROLES, USERS, NEXT
%token LPAREN, RPAREN, LBRACK, RBRACK, LBRACE, RBRACE
%token PLUS, COMMA, SEMICOLON, IMPLY, IFF, OR, AND, NOT
%token IN, NOTIN, EQ, NEQ, EOF, TRUE, FALSE
%token STATE, ACTION, INIT, TRANS, SPEC
%right IMPLY
%left IFF
%left OR
%left AND
%right NOT
%start contexts_eof actions_eof diagram_eof diagrams_eof 
%start lts_eof transitions_eof
%type <string list Formulas.contexts> contexts_eof
%type <string list Formulas.actions> actions_eof
%type <string list Formulas.diagram> diagram_eof
%type <string list Formulas.diagram list> diagrams_eof
%type <string list Formulas.lts> lts_eof
%type <string list Formulas.transitions> transitions_eof
%%

/* context formulas */

contexts_eof:
  contexts EOF               { $1 }
;

contexts:
  FALSE                      { Ctx_false }
| TRUE                       { Ctx_true }
| TYPES IN set               { Ctx_types $3 }
| ROLES IN set               { Ctx_roles $3 }
| USERS IN set               { Ctx_users $3 }
| TYPES NOTIN set            { Ctx_not (Ctx_types $3) }
| ROLES NOTIN set            { Ctx_not (Ctx_roles $3) }
| USERS NOTIN set            { Ctx_not (Ctx_users $3) }
| TYPES EQ STRING            { Ctx_types [$3] }
| ROLES EQ STRING            { Ctx_roles [$3] }
| USERS EQ STRING            { Ctx_users [$3] }
| TYPES NEQ STRING           { Ctx_not (Ctx_types [$3]) }
| ROLES NEQ STRING           { Ctx_not (Ctx_roles [$3]) }
| USERS NEQ STRING           { Ctx_not (Ctx_users [$3]) }
| NOT contexts               { Ctx_not $2 }
| contexts AND contexts      { Ctx_and ($1, $3) }
| contexts OR contexts       { Ctx_or ($1, $3) }
| contexts IFF contexts      { Ctx_iff ($1, $3) }
| contexts IMPLY contexts    { Ctx_imply ($1, $3) }
| LPAREN contexts RPAREN     { $2 }
;

/* action formulas */

actions_eof:
  actions EOF                { $1 }
;

actions:
  FALSE                      { Act_false }
| TRUE                       { Act_true }
| CLASSES IN set             { Act_classes $3 }
| PERMISSIONS IN set         { Act_permissions $3 }
| CLASSES NOTIN set          { Act_not (Act_classes $3) }
| PERMISSIONS NOTIN set      { Act_not (Act_permissions $3) }
| CLASSES EQ STRING          { Act_classes [$3] }
| PERMISSIONS EQ STRING      { Act_permissions [$3] }
| CLASSES NEQ STRING         { Act_not (Act_classes [$3]) }
| PERMISSIONS NEQ STRING     { Act_not (Act_permissions [$3]) }
| NOT actions                { Act_not $2 }
| actions AND actions        { Act_and ($1, $3) }
| actions OR actions         { Act_or ($1, $3) }
| actions IFF actions        { Act_iff ($1, $3) }
| actions IMPLY actions      { Act_imply ($1, $3) }
| LPAREN actions RPAREN      { $2 }
;

set:
  LBRACE set_elems RBRACE    { $2 }
;

set_elems:
                             { [] }
| non_null_set               { List.rev $1 }
;

non_null_set:
  STRING                     { [$1] }
| non_null_set COMMA STRING  { $3 :: $1 }
;

/* Diagrams */

diagram_eof:
  diagram EOF                           { $1 }
;

diagram:
  LBRACK elems RBRACK contexts          { $2, $4, Ctx_false, Act_false }
| LBRACK elems RBRACK contexts LBRACK contexts SEMICOLON actions RBRACK
                                        { $2, $4, $6, $8 }
;

elems:
                                        { [] }
| non_null_elems opt_semicolon          { List.rev $1 }
;

opt_semicolon:
                                        { }
| SEMICOLON                             { }
;

non_null_elems:
  arrow                                 { [$1] }
| non_null_elems SEMICOLON arrow        { $3 :: $1 }
;

arrow:
  contexts COMMA actions more           { $1, $3, $4 }
;

more:
                                        { false }
| PLUS                                  { true }
;

/* Diagram lists */

diagrams_eof:
  diagrams opt_semicolon EOF            { List.rev $1 }
;

diagrams:
  diagram                               { [$1] }
| diagrams SEMICOLON diagram            { $3 :: $1 }
;

lts_eof:
  type_def role_def user_def class_def permission_def
  init trans specs EOF
  {{types = $1;
    roles = $2;
    users = $3;
    classes = $4;
    permissions = $5;
    initial = $6;
    transition = $7;
    specifications = List.rev $8}}
;

type_def:
  STATE TYPES IN set                    { $4 }
;

role_def:
  STATE ROLES IN set                    { $4 }
;

user_def:
  STATE USERS IN set                    { $4 }
;

class_def:
  ACTION CLASSES IN set                 { $4 }
;

permission_def:
  ACTION PERMISSIONS IN set             { $4 }
;

init:
  INIT contexts                         { $2 }
;

trans:
  TRANS transitions                     { $2 }
;

specs:
                                        { [] }
| specs SPEC transitions                { $3 :: $1 }
;

/* Transitions */

transitions_eof:
  transitions EOF                       { $1 }
;

transitions:
  FALSE                                 { Tran_false }
| TRUE                                  { Tran_true }
| TYPES IN set                          { Tran_types $3 }
| ROLES IN set                          { Tran_roles $3 }
| USERS IN set                          { Tran_users $3 }
| TYPES NEXT IN set                     { Next_types $4 }
| ROLES NEXT IN set                     { Next_roles $4 }
| USERS NEXT IN set                     { Next_users $4 }
| CLASSES IN set                        { Tran_classes $3 }
| PERMISSIONS IN set                    { Tran_permissions $3 }
| TYPES NOTIN set                       { Tran_not (Tran_types $3) }
| ROLES NOTIN set                       { Tran_not (Tran_roles $3) }
| USERS NOTIN set                       { Tran_not (Tran_users $3) }
| TYPES NEXT NOTIN set                  { Tran_not (Next_types $4) }
| ROLES NEXT NOTIN set                  { Tran_not (Next_roles $4) }
| USERS NEXT NOTIN set                  { Tran_not (Next_users $4) }
| CLASSES NOTIN set                     { Tran_not (Tran_classes $3) }
| PERMISSIONS NOTIN set                 { Tran_not (Tran_permissions $3) }
| TYPES EQ STRING                       { Tran_types [$3] }
| ROLES EQ STRING                       { Tran_roles [$3] }
| USERS EQ STRING                       { Tran_users [$3] }
| TYPES NEXT EQ STRING                  { Next_types [$4] }
| ROLES NEXT EQ STRING                  { Next_roles [$4] }
| USERS NEXT EQ STRING                  { Next_users [$4] }
| TYPES NEXT EQ TYPES                   { Same_types }
| TYPES EQ TYPES NEXT                   { Same_types }
| ROLES NEXT EQ ROLES                   { Same_roles }
| ROLES EQ ROLES NEXT                   { Same_roles }
| USERS NEXT EQ USERS                   { Same_users }
| USERS EQ USERS NEXT                   { Same_users }
| CLASSES EQ STRING                     { Tran_classes [$3] }
| PERMISSIONS EQ STRING                 { Tran_permissions [$3] }
| TYPES NEQ STRING                      { Tran_not (Tran_types [$3]) }
| ROLES NEQ STRING                      { Tran_not (Tran_roles [$3]) }
| USERS NEQ STRING                      { Tran_not (Tran_users [$3]) }
| TYPES NEXT NEQ STRING                 { Tran_not (Next_types [$4]) }
| ROLES NEXT NEQ STRING                 { Tran_not (Next_roles [$4]) }
| USERS NEXT NEQ STRING                 { Tran_not (Next_users [$4]) }
| TYPES NEXT NEQ TYPES                  { Tran_not Same_types }
| TYPES NEQ TYPES NEXT                  { Tran_not Same_types }
| ROLES NEXT NEQ ROLES                  { Tran_not Same_roles }
| ROLES NEQ ROLES NEXT                  { Tran_not Same_roles }
| USERS NEXT NEQ USERS                  { Tran_not Same_users }
| USERS NEQ USERS NEXT                  { Tran_not Same_users }
| CLASSES NEQ STRING                    { Tran_not (Tran_classes [$3]) }
| PERMISSIONS NEQ STRING                { Tran_not (Tran_permissions [$3]) }
| NOT transitions                       { Tran_not $2 }
| transitions AND transitions           { Tran_and ($1, $3) }
| transitions OR transitions            { Tran_or ($1, $3) }
| transitions IFF transitions           { Tran_iff ($1, $3) }
| transitions IMPLY transitions         { Tran_imply ($1, $3) }
| LPAREN transitions RPAREN             { $2 }
;
