let report_error file_name lexbuf msg =
  let (lineno, start, finish) =
    Lex_formulas.lexeme_place lexbuf in
  let msg =
    Printf.sprintf
      "File \"%s\", line %d, characters %d-%d: %s"
      file_name lineno start finish msg in
  failwith msg

let report_parse_error file_name lexbuf =
  report_error file_name lexbuf "syntax error"

let open_in_or_stdin file_name =
  if file_name = "-" then
    stdin
  else
    open_in file_name

let read_lts file_name =
  let chan = open_in_or_stdin file_name in
  let lexbuf = Lexing.from_channel chan in
  Lex_formulas.init();
  try
    let lts =
      Parse_formulas.lts_eof Lex_formulas.token lexbuf in
    close_in chan;
    lts
  with Parsing.Parse_error ->
    close_in chan;
    report_parse_error file_name lexbuf
  | Failure s ->
    close_in chan;
    report_error file_name lexbuf s

let read_diagrams file_name =
  let chan = open_in_or_stdin file_name in
  let lexbuf = Lexing.from_channel chan in
  Lex_formulas.init();
  try
    let diagrams =
      Parse_formulas.diagrams_eof Lex_formulas.token lexbuf in
    close_in chan;
    diagrams
  with Parsing.Parse_error ->
    close_in chan;
    report_parse_error file_name lexbuf
  | Failure s ->
    close_in chan;
    report_error file_name lexbuf s

let read_diagram_from_string str =
  let lexbuf = Lexing.from_string str in
  Lex_formulas.init();
  try
    Parse_formulas.diagram_eof Lex_formulas.token lexbuf
  with Parsing.Parse_error ->
    report_parse_error "-" lexbuf
  | Failure s ->
    report_error "-" lexbuf s

let read_tran file_name =
  let chan = open_in_or_stdin file_name in
  let lexbuf = Lexing.from_channel chan in
  Lex_formulas.init();
  try
    let lts =
      Parse_formulas.transitions_eof Lex_formulas.token lexbuf in
    close_in chan;
    lts
  with Parsing.Parse_error ->
    close_in chan;
    report_parse_error file_name lexbuf
  | Failure s ->
    close_in chan;
    report_error file_name lexbuf s
