open Bdd
open Bdd_utils
open Formulas
open Descriptor

let list2bdd lookup list =
  let add bdd name =
    try
      disjoin bdd (lookup name)
    with Not_found -> failwith ("Name " ^ name ^ " not found") in
  List.fold_left add falsehood list

let make_identity lookup_in lookup_out list =
  let add bdd name =
    disjoin bdd (conjoin (lookup_in name) (lookup_out name)) in
  List.fold_left add falsehood list

let tran2bdd ~desc ~types ~roles ~users ~tran =
  let same_types =
    make_identity desc.lookup_in_type desc.lookup_out_type types in
  let same_roles =
    make_identity desc.lookup_in_role desc.lookup_out_role roles in
  let same_users =
    make_identity desc.lookup_in_user desc.lookup_out_user users in
  let rec recur tran =
    match tran with
      Tran_true -> truth
    | Tran_false -> falsehood
    | Tran_classes set -> list2bdd desc.lookup_cls set
    | Tran_permissions set -> list2bdd desc.lookup_perm set
    | Tran_types set -> list2bdd desc.lookup_in_type set
    | Next_types set -> list2bdd desc.lookup_out_type set
    | Same_types -> same_types
    | Tran_roles set -> list2bdd desc.lookup_in_role set
    | Next_roles set -> list2bdd desc.lookup_out_role set
    | Same_roles -> same_roles
    | Tran_users set -> list2bdd desc.lookup_in_user set
    | Next_users set -> list2bdd desc.lookup_out_user set
    | Same_users -> same_users
    | Tran_not tran -> complement (recur tran)
    | Tran_and (arg1, arg2) -> conjoin (recur arg1) (recur arg2)
    | Tran_or (arg1, arg2) -> disjoin (recur arg1) (recur arg2)
    | Tran_imply (arg1, arg2) -> implies (recur arg1) (recur arg2)
    | Tran_iff (arg1, arg2) ->
	let bdd1 = recur arg1 in
	let bdd2 = recur arg2 in
	conjoin (implies bdd1 bdd2) (implies bdd2 bdd1) in
  recur tran
